;*****************************************************************
;       NAP (extended sleep routine)
;
;       This routine sleeps the PIC for a period of time set by
;	16 bit value stored NAPHI:NAPLO. NAPHI:NAPLO is the number
;	of times the WDT times out before MAIN routine is executed
;	again.
;
;	Upon WDT timeout, processor wakes up very briefly and
;	decrements NAPHI:NAPLO. If NAPHI:NAPLO = 0 then the MAIN
;	loop is executed, otherwise the processor is returned to
;	sleep.
;
;	Input: none
;	Output: none
;
;	Requires: NAPHI & NAPLO registers defined, WDT active with an
;	appropriate timeout.
;
;	Executed by:
;
;	GOTO	NAP
;
;	DH MicroSystems, Inc.
;	WWW.DHMICRO.COM
;
;*****************************************************************
NAP	MOVLW	0x00		; high byte value of NAP counter
	MOVWF	NAPHI		; init NAPHI
	MOVLW	0x05		; low byte value of NAP counter
	MOVWF	NAPLO		; init NAPLO
	CLRWDT			; clear WDT
	SLEEP			; then goto sleep
	NOP

TEST	BTFSS	STATUS,NOT_TO	; check ~TO bit
	GOTO	CHKLO		; if clear goto CHKLO
	CLRWDT			; if set, wakeup occured for some
	SLEEP			; other reason (i.e. interrupt) &
	NOP			; is already taken care of (most
	GOTO	TEST		; likely by an interrupt handler)

CHKLO	MOVF	NAPLO,F
	BTFSC	STATUS,Z	; check NAPLO value
	GOTO	CHKHI		; if 0 check NAPHI value
	DECF	NAPLO,F		; else decrement NAPLO,
	CLRWDT			; clear WDT,
	SLEEP			; & go back to sleep
	NOP
	GOTO	TEST		; goto TEST upon wakeup

CHKHI	MOVF	NAPHI,F
	BTFSC	STATUS,Z	; check NAPHI value
	GOTO	MAIN		; if 0 time to wake up
	DECF	NAPHI,F		; else decrement NAPHI,
	MOVLW	0xFF		; set NAPLO to 0xFF,
	MOVWF	NAPLO
	CLRWDT			; clear WDT,
	SLEEP			; & go back to sleep
	NOP
	GOTO	TEST		; goto TEST upon wakeup

